"""
Module metric functions
"""

from collections import Counter
from statistics import mean, median_low, variance
from numpy import quantile

from mdav.common import MDAVType
from mdav.helper import get_columns, opencsvfile

# region Continous attributes


def get_dataset_columns_means(file, aux_file=None, headers=True,
                              correct=True, delimiter=";"):
    dataset = opencsvfile(file, aux_file, headers, correct, delimiter)
    columns = get_columns(dataset)
    result = []
    for i in range(len(columns)):
        if columns[i].type == MDAVType.continous:
            result.append(mean(columns[i].record))
        elif columns[i].type == MDAVType.ordinal:
            result.append(median_low(columns[i].record))
        elif columns[i].type == MDAVType.nominal:
            c = Counter(columns[i].record)
            result.append(c.most_common(1)[0][0])
        else:
            result.append(None)
    return result


def get_dataset_columns_variances(file, aux_file=None, headers=True,
                                  correct=True, delimiter=";"):
    dataset = opencsvfile(file, aux_file, headers, correct, delimiter)
    columns = get_columns(dataset)
    result = []
    for i in range(len(columns)):
        if columns[i].type == MDAVType.continous:
            result.append(variance(columns[i].record))
        else:
            result.append(None)
    return result


def get_dataset_columns_quantile50(file, aux_file=None, headers=True,
                                   correct=True, delimiter=";"):
    dataset = opencsvfile(file, aux_file, headers, correct, delimiter)
    columns = get_columns(dataset)
    result = []
    for i in range(len(columns)):
        if columns[i].type == MDAVType.continous:
            result.append(quantile(columns[i].record, .50))
        else:
            result.append(None)
    return result


def get_dataset_columns_quantile75(file, aux_file=None, headers=True,
                                   correct=True, delimiter=";"):
    dataset = opencsvfile(file, aux_file, headers, correct, delimiter)
    columns = get_columns(dataset)
    result = []
    for i in range(len(columns)):
        if columns[i].type == MDAVType.continous:
            result.append(quantile(columns[i].record, .75))
        else:
            result.append(None)
    return result

# endregion


# region Ordinal and nominal attributes


def get_dataset_mean_frequencies(file, aux_file=None, headers=True,
                                 correct=True, delimiter=";"):
    dataset = opencsvfile(file, aux_file, headers, correct, delimiter)
    columns = get_columns(dataset)
    result = []
    for i in range(len(columns)):
        if columns[i].type in [MDAVType.ordinal, MDAVType.nominal]:
            c = Counter(columns[i].record)
            result.append(mean(c.values()))
        else:
            result.append(None)
    return result


# endregion
