#!/usr/bin/env python3
"""
Command line interface for MDAV computations
"""

__author__ = "Javier Vilalta Bautista"
__version__ = "0.1.0"
__license__ = "MIT"


import argparse
import sys

from logzero import logger

from mdav import calc_mdav_generic, opencsvfile, savecsvfile


def main(args):
    """ Main entry point of the app """
    logger.info("Starting MDAV")
    logger.info(args)

    dataset = opencsvfile(args.input_file)
    logger.info(dataset)
    if args.algorithm == 'generic':
        private_dataset = calc_mdav_generic(dataset, args.parameter)
    elif args.algorithm == 'extended':
        raise ValueError('Unimplemented')
    else:
        raise ValueError('Invalid algoritm')
    savecsvfile(private_dataset, args.output_file)


# https://stackoverflow.com/a/14117511/149323
def check_positive(value):
    ivalue = int(value)
    if ivalue <= 0:
        raise argparse.ArgumentTypeError(
            "%s is an invalid positive int value" % value
        )
    return ivalue


if __name__ == "__main__":
    """ This is executed when run from the command line """
    parser = argparse.ArgumentParser(
        prog="MDAV",
        description="Anonimize a dataset using MDAV generic or MDAV "
        "with l-diversity."
    )

    # Optional argument for algorithm
    parser.add_argument(
        "-a", "--algorithm", action="store", dest="algorithm",
        choices=["generic", "extended"], default="generic",
        help="Specificies which algorithm to use, MDAV generic or MDAV "
        "extended with l-diversity"
    )

    # Optional argument for parameter
    parser.add_argument(
        "-p", "--parameter", action="store", dest="parameter",
        type=int, default=10,
        help="Specificies parameter k or l for MDAV"
    )

    # Optional argument for input file
    parser.add_argument(
        "-i", "--input", action="store",
        dest="input_file", default=sys.stdin.name
    )

    # Optional argument for output file
    parser.add_argument(
        "-o", "--output", action="store",
        dest="output_file", default=sys.stdout.name
    )

    # Optional verbosity counter (eg. -v, -vv, -vvv, etc.)
    parser.add_argument(
        "-v", "--verbose",
        action="count",
        default=0,
        help="Verbosity (-v, -vv, etc)")

    # Specify output of "--version"
    parser.add_argument(
        "--version",
        action="version",
        version="%(prog)s (version {version})".format(version=__version__))

    args = parser.parse_args()
    main(args)
