"""
Module for testing
"""

import csv
import os
import unittest

from logzero import logger

from mdav import calc_mdav_generic, opencsvfile
from mdav import savecsvfile

from mdav import (get_dataset_columns_means,
                  get_dataset_columns_variances)

DATA_FOLDER = 'data'


@unittest.skip("TestMDAV class skipped")
class TestMDAV(unittest.TestCase):
    """ This class computes MDAV with sample files """

    # region Auxiliary methods

    def setUp(self):
        self.generate_files = False
        self.path = DATA_FOLDER
        self.testfile = ''
        self.comparefile = ''
        self.k = 10

    def set_testfile(self, file):
        self.testfile = os.path.join(self.path, file)

    def set_comparefile(self, file):
        self.comparefile = os.path.join(self.path, file)

    # endregion

    def test_mdav_generic_sample1(self):
        self.mdav_generic('sample1.csv', self.k)

    def test_mdav_generic_US_census_1910(self):
        self.mdav_generic('ed007-pg001a.csv', self.k)

    def test_mdav_generic_US_census_1900(self):
        self.mdav_generic('ed009-pg019a.csv', self.k)

    # region Skipped tests

    @unittest.skip('Test skipped')
    def test_mdav_generic_continous(self):
        self.mdav_generic('continous.csv', self.k)

    @unittest.skip('Test skipped')
    def test_mdav_generic_ordinal_dispersed(self):
        self.mdav_generic('ordinal-dispersed.csv', self.k)

    @unittest.skip('Test skipped')
    def test_mdav_generic_ordinal_fullvalues(self):
        self.mdav_generic('ordinal-allvalues.csv', self.k)

    @unittest.skip('Test skipped')
    def test_mdav_generic_nominal(self):
        self.mdav_generic('nominal.csv', self.k)

    # endregion

    # region Compute functions

    def mdav_generic(self, testfile, k):
        file, extension = os.path.splitext(testfile)
        self.set_testfile(testfile)
        self.set_comparefile(f'{file}-mdav-generic-{k}{extension}')

        dataset = opencsvfile(self.testfile)
        private_dataset = calc_mdav_generic(dataset, k)

        if self.generate_files:
            logger.error("Generating result files for "
                         f"{self.__class__.__name__}, "
                         "test is not valid!")
            savecsvfile(private_dataset, self.comparefile)

        with open(self.comparefile, "r") as f:
            reader = csv.reader(f, delimiter=";")
            compare_dataset = list(reader)

        self.assertTrue(
            [str(i) for i in [y for x in private_dataset for y in x]] ==
            [y for x in compare_dataset for y in x]
        )

    # endregion


class TestMetric(unittest.TestCase):
    """ This class metric to compare sample files """
    # region Auxiliary methods

    def setUp(self):
        self.generate_files = True
        self.path = DATA_FOLDER
        self.testfile = ''
        self.comparefile = ''

    def set_testfile(self, file):
        self.testfile = os.path.join(self.path, file)

    def set_comparefile(self, file):
        self.comparefile = os.path.join(self.path, file)

    # end region

    def test_mean_sample1(self):
        self.calc_metric('mean', 'sample1.csv',
                         get_dataset_columns_means)

    def test_variance_sample1(self):
        self.calc_metric('variance', 'sample1.csv',
                         get_dataset_columns_variances)

    # region Compute functions

    def calc_metric(self, metric_name, testfile, metric_function):
        file, extension = os.path.splitext(testfile)
        self.set_testfile(testfile)
        self.set_comparefile(f'{file}-{metric_name}{extension}')
        result = [metric_function(self.testfile)]

        if self.generate_files:
            logger.error("Generating result files for "
                         f"{self.__class__.__name__}, "
                         "test is not valid!")
            savecsvfile(result, self.comparefile)

        with open(self.comparefile, "r") as f:
            reader = csv.reader(f, delimiter=";")
            compare_dataset = list(reader)

        self.assertTrue(
            ['' if None else str(i) for i in [y for x in result for y in x]] ==
            [y for x in compare_dataset for y in x]
        )

    # end region


if __name__ == "__main__":
    unittest.main(verbosity=2)
